@@ -26,14 +26,15 @@
 #
 
 import ldap.modlist as modlist
-
 import datetime
 import hubzero.config.passwords
 import hubzero.utilities.misc
+import hubzero.config.ldapconfig
 import hubzero.config.webconfig
 import hubzero.data.db
 import ldap
 
+
 def groupCleanup(db, groupname):
 	"""
 	We don't use foriegn keys, so messy cleanup here when you delete a group
@@ -143,7 +144,8 @@ def add_ldap_group(cn, groupDescription, gidNumber, members = {}):
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	dn = "cn=" + cn + ",ou=groups," + hubLDAPBaseDN
@@ -162,7 +164,8 @@ def delete_ldap_group(groupName):
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	dn = "cn=" + groupName + ",ou=groups," + hubLDAPBaseDN
@@ -241,7 +244,8 @@ def _ldapGroupExists(groupName):
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	resultid = l.search(hubLDAPBaseDN, ldap.SCOPE_SUBTREE, '(&(objectclass=posixGroup)(cn=' + groupName + '))', None)
@@ -257,7 +261,9 @@ def addusertogroup_ldap(userName, groupName):
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
-	l = ldap.initialize('ldap://localhost')
+	
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")	
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	# add the data to ldap
@@ -274,7 +280,9 @@ def deleteuserfromgroup_ldap(userName, groupName):
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
-	l = ldap.initialize('ldap://localhost')
+	
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")	
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	# add the data to ldap
@@ -325,3 +333,116 @@ def userInGroup(userName, groupName):
 
 	return False
 
+
+def syncgroups(hubname = "", purgeorphans = True, addgroupmemberships = True):
+	
+	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
+	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
+	dbName = hubzero.config.webconfig.getWebConfigDBName()
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
+	siteSuffix = hubzero.config.ldapconfig.getLDAPSiteSuffix()
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	
+	if purgeorphans:
+
+		# store of all gids from the DB and store them off for easy lookup
+		sql = "SELECT `cn` from jos_xgroups"
+		groupsFromDB = {}
+		for group in db.query(sql, None):
+			groupsFromDB[group.cn] = 1
+
+		l = ldap.initialize(ldapServer)
+		l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+		# get all groups from ldap
+		try:
+			r = l.search_s("ou=groups," + siteSuffix, ldap.SCOPE_SUBTREE, '(objectClass=posixGroup)',   ['cn'])
+
+		except ldap.NO_SUCH_OBJECT:
+			print "No groups found"
+			return
+	
+		# delete any group in ldap that isn't in the database
+		for dn,entry in r:
+			cn = str(entry['cn'][0])
+			if cn not in groupsFromDB:
+				print "deleting: " + cn
+				hubzero.utilities.group.delete_ldap_group(cn)	
+	
+
+	# grab complete informaton on all groups from db 
+	sql = "SELECT `cn`, `description`, `gidNumber` from jos_xgroups"
+	
+	for group in db.query(sql, None):
+	
+		if not group.description: group.description = ""
+ 
+		# get the groups membership if asked
+		groupMembers = []
+		if addgroupmemberships:
+			# get all group memberships for this group 
+			sql = """SELECT `p`.`username`, `g`.`cn`, `p`.`uidNumber`
+		    FROM jos_xgroups_members m 
+		    JOIN jos_xprofiles p on (p.uidNumber = m.uidNumber) 
+		    JOIN jos_xgroups g on (g.gidNumber = m.gidNumber)
+		    WHERE g.cn = %s"""
+			data = (group.cn)
+			
+			for userGroupMembership in db.query(sql, data):
+				groupMembers.append(str(userGroupMembership.uidNumber))
+
+		try:
+			print "syncing ldap group: " + group.cn
+			add_ldap_group(group.cn, group.description, group.gidNumber, groupMembers)
+		except ldap.ALREADY_EXISTS:
+			delete_ldap_group(group.cn)
+			add_ldap_group(group.cn, group.description, group.gidNumber, groupMembers)
+
+		if groupMembers:
+			print "  adding members:" + str(groupMembers)
+
+
+def _delete_ldap_users(args): 
+	
+	# by only deleting users in db, any users that exist only in ldap will be preserved
+	if not args.deleteorphans:
+		print "Orphaned users in ldap without a corresponding group in the database will be PRESERVED"
+		dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
+		dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
+		dbName = hubzero.config.webconfig.getWebConfigDBName()
+		db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+	
+		# grab all users from db 
+		sql = "SELECT `username`, `name`, `userPassword`, `uidNumber`, `homeDirectory`, `loginShell`, `gid`, gidNumber from jos_xprofiles"
+		
+		for user in db.query(sql, None):
+			print "deleting " + user.username
+			try:
+				hubzero.utilities.user.delete_ldap_user(user.username)
+			except ldap.NO_SUCH_OBJECT:
+				print " user not found"
+
+	else:
+		# grab all users from ldap
+		hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+		hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+		hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
+		siteSuffix = hubzero.config.ldapconfig.getLDAPSiteSuffix()
+		
+		ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+		l = ldap.initialize(ldapServer)
+		l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+		try:
+			r = l.search_s("ou=users," + siteSuffix, ldap.SCOPE_SUBTREE, '(objectClass=posixAccount)', ['uid'])
+
+			for dn,entry in r:
+				uid = str(entry['uid'][0])
+				hubzero.utilities.user.delete_ldap_user(uid)
+
+		except ldap.NO_SUCH_OBJECT:
+			print "No users in ou=users"
+	
\ No newline at end of file
