@@ -29,6 +29,7 @@ import datetime
 import hubzero.config.passwords
 import hubzero.utilities.group
 import hubzero.utilities.misc
+import hubzero.config.ldapconfig
 import hubzero.config.webconfig
 import hubzero.data.db
 import ldap
@@ -176,7 +177,8 @@ def _update_ldap_user_pw(username, uidNumber, password):
 	hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")	
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	dn = "uid=" + username + ",ou=users," + hubLDAPBaseDN
@@ -274,14 +276,15 @@ def add_ldap_user(uid,
                   gid, 
                   gidNumber, 
                   hubname = ""):
-
+	""" depecated, will eventaully replace with add_ldap_user2"""
+	
 	# get ldapManagerUserDN and PW
-
 	hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn", hubname)
 	hubLDAPAcctMgrPW= hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw", hubname)
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn", hubname)	
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 
 	dn = "uid=" + str(uid) + ",ou=users," + hubLDAPBaseDN
@@ -316,7 +319,8 @@ def delete_ldap_user(username, hubname = ""):
 	hubLDAPAcctMgrPW= hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw", hubname)
 	hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn", hubname)	
 
-	l = ldap.initialize('ldap://localhost')
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	l = ldap.initialize(ldapServer)
 	l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
 	dn = "uid=" + username + ",ou=users," + hubLDAPBaseDN
 
@@ -702,4 +706,53 @@ def add_ldap_user2(cn,
 		#	print e1
 		#	traceback.print_exc()
 		
+
+def syncusers(hubname = "", purgeorphans = True, syncattributes = True):
+	
+	dbPW = hubzero.config.webconfig.getWebConfigDBPassword()
+	dbUserName = hubzero.config.webconfig.getWebConfigDBUsername()
+	dbName = hubzero.config.webconfig.getWebConfigDBName()
+	ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+	db =  hubzero.data.db.MySQLConnection("localhost", dbName, dbUserName, dbPW)
+
+	# normally this would be accomplished with a complete purge and a readd, but we can't have our larger sites with an empty user database for
+	# the length of time it would take this script to run, so we do it this way
+	if purgeorphans:
+
+		print "Orphaned users in ldap without a corresponding group in the database will be DELETED"
+
+		# grab all users from db and store them off for easy lookup
+		sql = "SELECT `username` from jos_xprofiles"
+		usersFromDB = {}
+		for user in db.query(sql, None):
+			usersFromDB[user.username] = 1
+
+		hubLDAPAcctMgrDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerdn")
+		hubLDAPAcctMgrPW = hubzero.config.webconfig.getComponentParam("com_system", "ldap_managerpw")
+		hubLDAPBaseDN = hubzero.config.webconfig.getComponentParam("com_system", "ldap_basedn")
+		siteSuffix = hubzero.config.ldapconfig.getLDAPSiteSuffix()
+
+		#grab all users from ldap
+		ldapServer = hubzero.config.webconfig.getComponentParam("com_system", "ldap_primary")
+		l = ldap.initialize(ldapServer)
+		l.simple_bind_s(hubLDAPAcctMgrDN, hubLDAPAcctMgrPW)
+
+		r = l.search_s("ou=users," + siteSuffix, ldap.SCOPE_SUBTREE, '(objectClass=posixAccount)', ['uid'])
+
+		for dn,entry in r:
+			uid = str(entry['uid'][0])
+			
+			if uid not in usersFromDB:
+				print "uid: " + str(uid) + " not in database, must delete"
+				try:
+					hubzero.utilities.user.delete_ldap_user(user.username)
+				except ldap.NO_SUCH_OBJECT:
+					print " user not found"
+
+	# now do a mass add of all users from db	
+	sql = "SELECT `username`, `name`, `userPassword`, `uidNumber`, `homeDirectory`, `loginShell`, `gid`, gidNumber from jos_xprofiles"
+	
+	for user in db.query(sql, None):
+		print "syncing user:" + user.username
+		syncUser(user.username, hubname, syncattributes)
 	
\ No newline at end of file
